@@ -1,5 +1,6 @@
 #A number of useful helper functions
 #added January, 2012
+#most are public, some are local just for me
 
 #a dummy function to allow the help to find misc
 "psych.misc" <-
@@ -41,8 +42,7 @@ function(x,digits=2,use="pairwise",method="pearson") {
 #modified August 10, 2012 to print just 100 times. 
 "progressBar" <- 
 function(value,max,label=NULL) {
-if(class(stdout())[1]=="terminal")  #only print to the screen, not to a file
- { pc <- round(100 * value/max)
+if(inherits(stdout()[1],"terminal")) { pc <- round(100 * value/max)  #only print to the screen, not to a file
 if(ceiling(100 * value/max)==floor(100 * value/max)) {
 width <- 100
 char="."
@@ -60,6 +60,9 @@ nb <- round(width * value/max )
 
 "reflect" <- 
 function(f,flip=NULL) {
+if(is.null(names(f))) {temp <- f
+    f <-list()
+    f$loadings <- temp}
 rnames <- colnames(f$loadings)
 rnames[flip] <- paste(rnames[flip],'(R)',sep="")
 flipper <- rep(1,ncol(f$loadings))
@@ -72,18 +75,24 @@ if(!is.null(f$scores)) f$scores <- f$scores %*% flip
 if(!is.null(f$Phi)) f$Phi <- flip %*% f$Phi %*% t(flip)
 return(f)
 }
+
+
+
+
 #and futher patched May 10, 2017 to get the right denominator (I was dividing by n-1 - lag instead of n-lag
 #patched March 1, 2017 to get the df right for lags 
 #developed January 10, 2012 to find the mean square of successive differences
 #see Von Neuman et al. 1941
+#added the minus 1 to the case of a single vector.  Sept 21, 2017
+#the matrix version was correct, just not the single case version
 "mssd" <- function(x,group=NULL,lag=1,na.rm=TRUE) {
 if(is.null(group)) {
-if(is.vector(x)) { result <- sum(diff(x,lag=lag,na.rm=na.rm)^2,na.rm=na.rm)/(sum(!is.na(x))-lag)} else {
+if(is.vector(x)) { result <- sum(diff(x,lag=lag,na.rm=na.rm)^2,na.rm=na.rm)/(sum(!is.na(x)) -lag )} else {
 x <- as.matrix(x)
 if(NCOL(x) == 1) {
   result <- sum(diff(x,lag=lag,na.rm=na.rm)^2,na.rm=na.rm)/(sum(!is.na(x))-lag)
   } else {
-n <- colSums(!is.na(x)) -1 -lag
+n <- colSums(!is.na(x))  -lag
 result <- colSums(diff(x,lag=lag,na.rm=na.rm)^2,na.rm=na.rm)/n}
 } } else {
 x <- as.matrix(x) #added 26/5/14
@@ -170,13 +179,6 @@ x.df <- data.frame(x,rx,x2,rx2,xg)
 return(x.df)}
 
 
-"build.html.help" <- function(p="psych",fn = "/Volumes/Test/psych/man",out="/Volumes/Test/help/") {        
-    
-     db <- list.files(fn)
-     for (f in db) {tools::Rd2HTML(paste(fn,db[f]),out=paste(out,db[f]),package=p)
-     } 
-     }
-        
 
     
 #shannon complexity index
@@ -206,8 +208,19 @@ test.all <- function(p) {
  detach(ob,character.only=TRUE)
 }
 
+#lookup a set of items from a bigger set
+lookupItem <- function(x,y) {
+n.look <- NROW(x)
+possible <- list()
+y <- as.character(y)
+x <- as.character(x)
+for(i in 1:n.look){
+ temp <- grep(x[i],y)
+ if(length(temp)>0) possible[i]<- temp
 
-
+}
+return(possible)
+}
   
   
   #lookup which x's are found in y[c1],return matches for y[]
@@ -215,16 +228,21 @@ test.all <- function(p) {
 function(x,y,criteria=NULL) {
 if (is.null(criteria)) {temp <- match(x,rownames(y))} else {
      temp <- match(x,y[,criteria])}
- y <- (y[temp[!is.na(temp)],,drop=FALSE])
+     if(any(!is.na(temp))) {
+ y <- (y[temp[!is.na(temp)],,drop=FALSE]) } else {y <- NA}
 return(y)}
  
  #use lookup to take fa/ic output and show the results 
  #modified July 4, 2017 to allow for omega output as well
+ #modified June 23, 2018 to limit to top n items per factor and abs(loading) > cut
 "fa.lookup"  <-
-   function(f,dictionary,digits=2) {
-    f <- fa.sort(f)
-   if(length(class(f)) > 1){ value <- class(f)[2] } else {value <- "none"}
-   
+   function(f,dictionary=NULL,digits=2,cut=.0,n=NULL,sort=TRUE) {
+    if(sort) {f <- fa.sort(f)}
+    none <- NA 
+   if(length(class(f)) > 1){ obnames <- cs(omega, fa, principal, iclust, none)
+     value <- inherits(f, obnames, which=TRUE)
+			   if (any(value > 1)) {value <- obnames[which(value >0)]} else {value <- "none"}}
+  
    switch(value,
     
     omega = {f <- f$schmid$sl
@@ -242,28 +260,79 @@ return(y)}
     none = {f <- f
             h2 <- NULL})
     
-    
+    n.fact <- NCOL(f)
 
    ord <- rownames(f)
-     
-   contents <- lookup(rownames(f),dictionary)
+   old.names <- ord
+   ord <- sub("-","",ord)
+   rownames(f) <- ord
+   if(!is.null(dictionary))  { 
+   contents <- lookup(rownames(f),dictionary)} else {message("fa.lookup requires a dictionary, otherwise just use fa.sort")}
    if(!is.null(h2)) {results <- data.frame(round(unclass(f),digits=digits),com=round(com,digits=digits),h2=round(h2,digits=digits))} else {
    results <- data.frame(round(unclass(f),digits=digits))}
+   
    results <- merge(results,contents,by="row.names",all.x=TRUE,sort=FALSE)
    rownames(results) <- results[,"Row.names"]
    results <- results[ord,-1]  #now put it back into the correct order
+   rownames(results) <- old.names
+   if(!is.null(n)) {
+     rn <-rownames(results)
+     results <- cbind(results,rn)
+  	 f2c <- table(apply(abs(results[1:n.fact]),1,which.max))   #which column is the maximum value
+   	 k <- 1
+   	 j <- 1
+   	 for(i in 1:n.fact) {
+   		 results[k:(k+min(n,f2c[i])),] <-  results[j:(j+ min(n,f2c[i])),]
+   		 k <- (k+min(n,f2c[i]))  
+   		 j <- j + f2c[i]   }
+    	results <- results[1:(k-1),]
+    	rownames(results) <- results[,"rn"]
+    	results <- results[,-NCOL(results)]
+    }
+    
+if(cut > 0) {
+   r.max <- apply(abs(results[,1:n.fact]),1,max)
+   results <- results[abs(r.max) > cut,]
+   }
 return(results)}
   
 
 
 
- 
+ #revised 07/07/18 to add the cluster option
+ #revised 12/07/18 to allow for simple matrix input
+ #read a matrix, return a matrix
+ #read a list, return a list
   "fa.organize" <- 
-function(fa.results,o=NULL,i=NULL,cn=NULL) {
+function(fa.results,o=NULL,i=NULL,cn=NULL,echelon=TRUE,flip=TRUE) {
+if(is.null(names(fa.results)) )  {temp <- fa.results   #the matrix form          
+                 if(flip) {
+                 total.load <-colSums(temp)
+                 flipper <- sign(total.load)
+                 flipper[flipper==0] <-1 
+                 temp <- t( t(temp) * flipper ) }
+                 if(!is.null(o)) {temp <- temp[,o]}
+                 if(!is.null(i)) {temp <-temp[i,]}
+                 fa.results <- temp } else { # the list form 
+     if(echelon & is.null(o) ) {temp <- apply(abs(  fa.results$loadings),1,which.max)
+     nf <- ncol(fa.results$loadings)
+     nvar <- nrow(fa.results$loadings)
+  o <- 1:nf
+  k <- 1
+  o[k] <- temp[k]
+  for (ki in 2:nvar) {if (!(temp[ki] %in% o[1:k])) {o[k+1] <- temp[ki]
+    k <- k + 1}  }
+    }
+  if(flip) {
+        total.load <- colSums(fa.results$loadings)
+       flipper <- sign(total.load)
+        flipper[flipper==0] <-1 } else { flipper <- rep(1,NCOL(fa.results$loadings)) }
+   fa.results$loadings <- t(t(fa.results$loadings) * flipper)                      
+
   if(!is.null(o)) {fa.results$loadings <- fa.results$loadings[,o]
-       fa.results$Structure <- fa.results$Structure[,o]
-       fa.results$weights <- fa.results$weights[,o]
-       fa.results$valid <- fa.results$valid[o]
+      flipper <- flipper[o] 
+       fa.results$Structure <- t(t(fa.results$Structure[,o]) * flipper)
+       fa.results$valid <- t(t(fa.results$valid[o])*flipper)
        fa.results$score.cor <- fa.results$score.cor[o,o]
        fa.results$r.scores <- fa.results$r.scores[o,o]
        fa.results$R2 <- fa.results$R2[o]
@@ -273,7 +342,8 @@ function(fa.results,o=NULL,i=NULL,cn=NULL) {
    fa.results$Structure <- fa.results$Structure[i,]
        fa.results$weights <- fa.results$weights[i,]
        fa.results$complexity=fa.results$complexity[i]
-       fa.results$uniquenesses <- fa.results$uniquenesses[i]}
+       fa.results$uniquenesses <- fa.results$uniquenesses[i]} 
+       }
   return(fa.results)
   }
   
@@ -315,6 +385,30 @@ function(fa.results,o=NULL,i=NULL,cn=NULL) {
   "item.lookup" <- 
 function (f,m, dictionary,cut=.3, digits = 2) {
     f <- fa.sort(f)
+    none<- NULL   #A strange requirement of R 4.0
+     if(length(class(f)) > 1){ obnames <- cs(omega, fa, principal, iclust, none)
+     value <- inherits(f, obnames, which=TRUE)
+			   if (any(value > 1)) {value <- obnames[which(value >0)]} else {value <- "none"}} else {value <- "none"}
+  old.names <- NULL
+   switch(value,
+    
+    omega = {f <- f$schmid$sl
+            h2 <- NULL
+            old.names <- rownames(f)
+            rownames(f) <- sub("-","",old.names)},
+    fa    = {
+             h2 <- f$communality
+              com <- f$complexity
+              f <- f$loading        },
+    principal= {
+                h2 <- f$communality
+              com <- f$complexity
+              f <- f$loading},
+    iclust = {f <- f$loadings
+              h2 <- NULL},
+    none = {f <- f
+            h2 <- NULL})
+    
     if (!(is.matrix(f) || is.data.frame(f))) {
         h2 <- f$communality
         com <- f$complexity
@@ -344,14 +438,35 @@ function (f,m, dictionary,cut=.3, digits = 2) {
         sort = FALSE)
     rownames(results) <- results[, "Row.names"]
     results <- results[ord, -1]
-    res <- results[0,]  #make an empty data frame of the structure of results
-    for (i in 1:nfact) { temp <-results[abs(results[,i]) > cut,]
-     ord <- order(temp[,"means"])
-     res <- rbind(res,temp[ord,])
-     }
+    if(!is.null(old.names)) rownames(results) <- old.names
+    res <- results
+  #   res <- results[0,]  #make an empty data frame of the structure of results
+#     for (i in 1:nfact) { temp <- results[abs(results[,i]) > cut,]
+#      ord <- order(temp[,"means"])
+#      res <- rbind(res,temp[ord,])
+#      }
     return(res)
 }
 
+setCorLookup<- function(x,dictionary=NULL,cut=0,digits=2,p=.05) {
+coef <- x$coefficients
+probs <- x$Probability
+labels <- dictionary[rownames(coef),,drop=FALSE]
+
+coef[probs > p] <- NA
+result <- list()
+nvar <- NCOL(coef)
+for (i in 1:nvar) {
+ord <- order(abs(coef[,i]),decreasing=TRUE)
+temp <- cbind(coef=round(coef[ord,i],digits),labels[ord,])
+result[[i]] <- data.frame(temp[!is.na(temp[,1]),])
+}
+names(result) <- colnames(coef)
+result
+}
+
+
+
 
 
 "falsePositive" <- function(sexy=.1,alpha=.05,power=.8) {
@@ -359,6 +474,13 @@ pf <- alpha * (sexy)
 vp <- power * (1-sexy)
 pf/(pf+vp)}
 
+"build.html.help" <- function(p="psych",fn = "/Volumes/Test/psych/man",out="/Volumes/Test/help/") {        
+    
+     db <- list.files(fn)
+     for (f in db) {tools::Rd2HTML(paste(fn,db[f]),out=paste(out,db[f]),package=p)
+     } 
+     }
+        
 
 
 "bullseye" <- function(x,y,n) {
@@ -464,7 +586,7 @@ pretty}
 
 #October 25, 2016
 "char2numeric" <- function(x) {
- nvar <- ncol(x)
+ nvar <- NCOL(x)
  for(i in 1:nvar) {   
         if(!is.numeric(x[[i]] ))  {
                                   if(is.factor(unlist(x[[i]])) | is.character(unlist(x[[i]]))) {  x[[i]] <- as.numeric(x[[i]]) 
@@ -472,52 +594,40 @@ pretty}
               } 
               invisible(x)} 
 
-#this just shows if it is a matrix
+#this just shows if it is a matrix is symmetric and has diagonals of 1
 "isCorrelation" <-  function(x) {value <- FALSE
   if(NROW(x) == NCOL(x)) {
   if( is.data.frame(x)) {if(isSymmetric(unname(as.matrix(x)))) { value <- TRUE}} else {if(isSymmetric(unname(x))) {value <- TRUE}}}
-return(value)}
-
-
-#will only work for data.frames (not matrices)
-dfOrder <- function(object,columns=NULL) {if(is.null(columns)) columns <- 1:ncol(object)
- nc <- length(columns)
- cn <-colnames(object)
-  temp <- rep(1,nc)
-  if(is.character(columns)) {  #treat character strings 
-    temp [strtrim(columns,1)=="-"] <- -1
-    if(any(temp < 0) )  {columns <- sub("-","",columns) }
-    columns <- which(colnames(object) %in% columns)
-   } else {temp <- sign(columns)
-      columns <- abs(columns)
-    }
-    
-  if(any(temp < 0)) {object [temp <0] <- -object[temp < 0 ]
-   }
-  bad <- FALSE
-  if(!is.data.frame(object)) stop("I am sorry, I can only sort data.frames.")
-   #temp <- colnames(object)
-   templist <- list()
-   increase <- sign(temp)
-   for (i in 1:nc) {if(!is.numeric(object[[(columns[i])]])) bad<- TRUE}
-    if(bad) {if(any(increase < 0)) {stop("I am sorry, I can not sort data.frames with characters in decreasing order") } else {
-     for(i in 1:nc) { templist[[i]] <- columns[[i]]   }
-     ord <- with(object,do.call(order,templist))
-     object <- object[ord,]
-          }
-          } else { 
+  value <- value && isTRUE(all.equal(prod(diag(as.matrix(x))),1) )
+  value <- value && isTRUE((min(x)>= -1) & (max(x) <= 1))
+  return(value)}
+  
+  #this just shows if it is a symmetric matrix
+"isCovariance" <-  function(x) {value <- FALSE
+  if(NROW(x) == NCOL(x)) {
+  if( is.data.frame(x)) {if(isSymmetric(unname(as.matrix(x)))) { value <- TRUE}} else {if(isSymmetric(unname(x))) {value <- TRUE}}}
+ # value <- value && isTRUE(all.equal(prod(diag(as.matrix(x))),1) )  #don't check for diagonal of 1
+  return(value)}
+  
   
-    for (i in 1:nc) {
-   		 templist[[i]] <- as.name(cn[columns[i]])
-    		 object[,(columns[i])] <- increase[i] *object[,(columns[i])]
-                    }
 
-   ord <- with(object,do.call(order,templist))
-   object <- object[ord,]
-   for(i in 1:nc) {object[,(columns[i])] <- increase[i] *object[,(columns[i])]}
-   }
-    return(object)
-    }
+
+#cs is taken from Hmisc:::Cs
+cs <- function(...) {as.character(sys.call())[-1]}
+#acs is modified to produce a single string
+acs <- function(...) {gsub(",","",toString(sys.call()[-1]))}
+
+fromTo <- function(data,from,to=NULL) {cn <- colnames(data)
+    if(is.null(to)) {to <- from[2]
+       from <- from[1]}
+  from <- which(cn == as.character(from))
+ to =  which(cn == as.character(to))
+  select <- from:to
+  return(data[select])}
+  
+  
+
+
     
     
  
