@@ -18,10 +18,21 @@
     on.exit(par(old.par))  #set them back
  
  #Figure out what type of input we are drawing   (done July 2, 2018)
+ #fixed for R 4.0.0  December 11, 2019
   extend<- FALSE
-  if(length(class(om.results)) > 1) {result <- class(om.results) [2]} else {result <- "extend"}
+  
+#which kind of input are we drawing?
+    if(length(class(om.results)) > 1)  {
+      omegaSem <- omegaDirect <- omega <- NULL  #strange fix to R 4,0,0 compiler
+    names <- cs(extend, omegaSem, omegaDirect, omega)
+    value <- inherits(om.results,names,which=TRUE)  # value <- class(x)[2]
+    if(any(value > 1) ) { result <- names[which(value > 0)]} else {result <- "other"}
+    
+    } else {result <- "extend"}    #just a raw matrix
+    
+#dispatch to the right option
     switch(result,
-      extend = {extend <- TRUE
+extend = {extend <- TRUE
                #class(om.results)[2] <- "omega"
                factors <- om.results
                nvar <- num.var <- nrow(factors)
@@ -30,7 +41,7 @@
                    temp2 <- factors[,1,drop=FALSE]     #added the drop option November 4, 2018
           factors <- cbind(g=temp2[rownames(temp),1],temp)}  #added column 1
             }, 
-      omegaSem = {
+omegaSem = {
     #did we do an omegaSem or just an omegaFromSem?
     if(is.null(om.results$omega.efa$cfa.loads)) {cfa.loads <- om.results$cfa.loads} else {cfa.loads <- om.results$omega.efa$cfa.loads}
       #  factors <- as.matrix(om.results$omega.efa$cfa.loads[,2:ncol(om.results$omega.efa$cfa.loads)])
@@ -43,15 +54,16 @@
         sl=TRUE
         main <- "Omega from SEM" 
     },
-    omegaDirect = {factors <- om.results$loadings
+    
+ omegaDirect = {factors <- om.results$loadings
      nvar <- num.var <- nrow(factors)
      num.factors <- ncol(factors) -1
                   if(sort) {temp  <- fa.sort(factors[,-1])
                    temp2 <- factors[,1]
           factors <- cbind(g=temp2[rownames(temp)],temp)} },
     
-    omega ={
-        if(extend) class(om.results)[2] <- "omega"
+omega ={
+        if(extend) class(om.results) <- c("psych","omega")
  if(sort) om.results <- fa.sort(om.results)   #usually sort, but sometimes it is better not to do so
           
  if (sl) {factors <- as.matrix(om.results$schmid$sl) 
@@ -64,43 +76,12 @@
         nvar <- num.var <- dim(factors)[1]   #how many variables?
    if (sl ) {num.factors <- dim(factors)[2] - 1 - (!extend) *3 } else {num.factors <- dim(factors)[2]
        }
- }
+ },
+ other ={warning("I am sorry, I don't know how to diagram this input")}
       
     )
+if(result !="other") {#skip to the end if we don' know what we are doing
  
- #removed July 2, 2018 when we added the switch option 
-#   if((length(class(om.results)) > 1)  && ( (class(om.results)[2] == "extend")) ) {extend <- TRUE
-#         class(om.results)[2] <- "omega"} else {extend <- FALSE}
-# #  if((length(class(om.results)) > 1)  && ( (class(om.results)[2] == "omegaSem")) ) {
-# #     #did we do an omegaSem or just an omegaFromSem?
-# #     if(is.null(om.results$omega.efa$cfa.loads)) {cfa.loads <- om.results$cfa.loads} else {cfa.loads <- om.results$omega.efa$cfa.loads}
-# #       #  factors <- as.matrix(om.results$omega.efa$cfa.loads[,2:ncol(om.results$omega.efa$cfa.loads)])
-# #       class(cfa.loads) <- c("psych","omegaSem")
-# #       if(sort) cfa.loads <- fa.sort(cfa.loads)
-# #       factors <- as.matrix(cfa.loads)
-# #         gloading <- cfa.loads[,1,drop=FALSE]
-# #         nvar <- num.var <- nrow(gloading)
-# #         num.factors <- ncol(factors) -1
-# #         sl=TRUE
-# #         main <- "Omega from SEM" 
-#            } else {
-#         if(extend) class(om.results)[2] <- "omega"
-#  if(sort) om.results <- fa.sort(om.results)   #usually sort, but sometimes it is better not to do so
-#           
-#  if (sl) {factors <- as.matrix(om.results$schmid$sl) 
-#          
-#          if(is.null(main)) {main <- "Omega with Schmid Leiman Transformation" }
-#          } else {factors <- as.matrix(om.results$schmid$oblique)
-#           if(is.null(main)) {main <- "Hierarchical (multilevel) Structure" }
-#          }
-#        gloading <- om.results$schmid$gloading
-#         nvar <- num.var <- dim(factors)[1]   #how many variables?
-#    if (sl ) {num.factors <- dim(factors)[2] - 1 - (!extend) *3 } else {num.factors <- dim(factors)[2]
-#        }
-#  }
-
-
-
 #now draw the figure
    
    e.size <- e.size * 10/ nvar   #this is an arbitrary setting that seems to work
@@ -159,4 +140,5 @@
   if (errors) {for (v in 1:nvar) {
        dia.self(location=var.rect[[v]],scale=.5,side=side)  }
        }
+    } #end of normal case
 }                  
\ No newline at end of file
