@@ -2,7 +2,11 @@
 function(data,x,y,use="pairwise",method="pearson")  {
  cl <- match.call()
    if(!isCorrelation(data)) {n.obs <- dim(data)[1]
-                    data <- cor(data,use=use,method=method) }
+    if(!missing(x) & !missing(y)) {if(!is.character(x) ) x <- colnames(data)[x]
+       if(!is.character(y) ) y <- colnames(data)[y]
+        data <- cor(data[,c(x,y)],use=use,method=method)
+         } else {if(is.null(dim(data))) stop("Specify the rows for data (use , for all rows)")
+                    data <- cor(data,use=use,method=method) }}
    m <- as.matrix(data)
  if(missing(x) & missing(y)) {X.resid <- -(solve(m))  #this is thus the image covariance matrix
     diag(X.resid) <- 1/(1- smc(m))    #adjust the diagonal to be 1/error
@@ -21,4 +25,5 @@ function(data,x,y,use="pairwise",method="pearson")  {
      	}
      #modified March 23 to use cov2cor instead of the sd line.  This makes the diagonal exactly 1.
      #05/08/17  Completely rewritten to be easier to use and follow for the case of complete partials 
+     #modified 03/19/19 to just choose the items to correlate instead of entire matrix
 
