@@ -96,9 +96,11 @@ ans
 #revised 16/6/18 to cover the problem of 0 values in cells
 polyF <- function(rho,rc,cc,tab) {  #doesn't blow up in the case of 0 cell entries  added 16/6/18
       P <- polyBinBvn(rho, rc, cc) 
-      lP <- log(P)
+     P[P <=0] <- NA   #added 18/2/9
+     lP <- log(P)
       lP[lP == -Inf] <- NA
-       -sum(tab * lP,na.rm=TRUE) }  #the  criterion to be minimized
+       lP[lP == Inf] <- NA
+       -sum(tab * lP,na.rm=TRUE)  }  #the  criterion to be minimized
        
        
 "wtd.table" <- function(x,y,weight) {
@@ -109,15 +111,17 @@ return(tab)
 
     
  
-  #modified 10/8/14 to create missing values when there are no cell entries
-  #modified 3/6/14 to create missing values when the data are hopeless
-
+#modified 10/8/14 to create missing values when there are no cell entries
+#modified 3/6/14 to create missing values when the data are hopeless
+#modified 06/2/18 for the case of empty cells
  "polyc" <-     #uses the tableFast function instead of tableF
 function(x,y=NULL,taux,tauy,global=TRUE,weight=NULL,correct=correct,gminx,gmaxx,gminy,gmaxy) {
   if(is.null(weight )) {tab <- tableFast(x,y,gminx,gmaxx,gminy,gmaxy)
    }   else {tab <- wtd.table(x,y,weight)}  #need to specify minx and maxx somehow
    fixed <- 0 
   tot <- sum(tab)
+  if(tot ==0) {result <- list(rho=NA,objective=NA,fixed=1)
+               return(result)} #we have no data for this cell   05/02/18
   tab <- tab/tot
    if(correct > 0) {if(any(tab[]==0)) {fixed <- 1
                  tab[tab==0] <- correct/tot }} #moved from below 16.6.22
@@ -147,12 +151,13 @@ function(x,y=NULL,taux,tauy,global=TRUE,weight=NULL,correct=correct,gminx,gmaxx,
    
   return(result)
   }
-  
+##########
 
 #We have dropped option to use John Fox's polycor package, so we don't need the options
 #function(x,smooth=TRUE,global=TRUE,polycor=FALSE,ML = FALSE, std.err = FALSE,weight=NULL,correct=.5,progress=TRUE,na.rm=TRUE,delete=TRUE) {
+#12/25/19 added the ability to have a y set of  variables as well
 "polychoric" <- 
-function(x,smooth=TRUE,global=TRUE,polycor=FALSE,ML = FALSE, std.err = FALSE,weight=NULL,correct=.5,progress=TRUE,na.rm=TRUE,delete=TRUE)  {
+function(x,y=NULL,smooth=TRUE,global=TRUE,polycor=FALSE,ML = FALSE, std.err = FALSE,weight=NULL,correct=.5,progress=TRUE,na.rm=TRUE,delete=TRUE)  {
 #function(x,smooth=TRUE,global=TRUE,polycor=FALSE,weight=NULL,correct=.5,progress=TRUE,na.rm=TRUE,delete=TRUE) {
 #if(!require(parallel)) {message("polychoric requires the parallel package.")}
 #declare these next two functions to be local inside of polychoric
@@ -163,7 +168,8 @@ if(std.err) message("The std.error option has been removed from the polychoric f
 
 
 
-myfun <- function(x,i,j,gminx,gmaxx,gminy,gmaxy) {polyc(x[,i],x[,j],tau[,i],tau[,j],global=global,weight=weight,correct=correct,gminx=gminx,gmaxx=gmaxx,gminy=gminy,gmaxy=gmaxy) }
+myfun <- function(x,y,i,j,gminx,gmaxx,gminy,gmaxy) {polyc(x[,i],x[,j],tau[,i],tau[,j],global=global,weight=weight,correct=correct,gminx=gminx,gmaxx=gmaxx,gminy=gminy,gmaxy=gmaxy) }
+myfuny <- function(x,y,i,j,gminx,gmaxx,gminy,gmaxy,tauy) {polyc(x[,i],y[,j],tau[,i],tauy[,j],global=global,weight=weight,correct=correct,gminx=gminx,gmaxx=gmaxx,gminy=gminy,gmaxy=gmaxy) }
 
 matpLower <- function(x,nvar,gminx,gmaxx,gminy,gmaxy) {
 k <- 1
@@ -174,7 +180,8 @@ for(i in 2:nvar) {for (j in 1:(i-1)) {
    jl [k] <- j
    k<- k+1}
    }
-poly <- mcmapply(function(i,j) myfun(x,i,j,gminx=gminx,gmaxx=gmaxx,gminy=gminy,gmaxy=gmaxy) , il,jl) 
+poly <- mcmapply(function(i,j) myfun(x,y=NULL,i,j,gminx=gminx,gmaxx=gmaxx,gminy=gminy,gmaxy=gmaxy) , il,jl) 
+#poly <- mapply(function(i,j) myfun(x,i,j,gminx=gminx,gmaxx=gmaxx,gminy=gminy,gmaxy=gmaxy) , il,jl) 
 #debugging, we turn off the mcmapply function and do it by hand
 # browser()
 # ppl <- list()
@@ -193,11 +200,52 @@ if((fixed > 0) && ( correct > 0)) { warning(fixed ," cells were adjusted for 0 v
 return(mat)} else {
 warning("Something is wrong in polycor ")
 return(poly)
-
+#never actually gets here
 stop("we need to quit because something was seriously wrong.  Please look at the results")} 
 }
 
+matpxy <- function(x,y,nvar,nvar.y,gminx,gmaxx,gminy,gmaxy,tauy) {
 
+
+if(!is.matrix(tauy)) tauy <- matrix(tauy,ncol=nvar.y)
+#make the lists that are passed to myfuny
+k <- 1
+il <- vector()
+jl <- vector()
+for(i in 1:nvar) {for (j in 1:(nvar.y)) {
+   il[k] <- i
+   jl [k] <- j
+   k<- k+1}
+   }
+ 
+   
+
+#poly <- mapply(function(i,j) myfuny(x,y,i,j,gminx=gminx,gmaxx=gmaxx,gminy=gminy,gmaxy=gmaxy,tauy) , il,jl) 
+poly <- mcmapply(function(i,j) myfuny(x,y,i,j,gminx=gminx,gmaxx=gmaxx,gminy=gminy,gmaxy=gmaxy,tauy) , il,jl) 
+ 
+
+# browser()
+# ppl <- list()
+# for (i in 2:nvar) {for (j in 1:(i-1)) {ppl[[i+j]] <- myfun(x,i,j,gminx=gminx,gmaxx=gmaxx,gminy=gminy,gmaxy=gmaxy) } }
+
+#now make it a matrix
+mat <-  matrix(NA,ncol=nvar,nrow=nvar.y)
+if(length(dim(poly)) == 2) {
+mat<-  as.numeric(poly[1,]) #first row of poly is correlation, 2nd the fit
+
+ fixed <- as.numeric(poly[3,])
+
+fixed <- sum(fixed) 
+if((fixed > 0) && ( correct > 0)) { warning(fixed ," cells were adjusted for 0 values using the correction for continuity. Examine your data carefully.")}
+
+return(mat)} else {
+warning("Something is wrong in polycor ")
+return(poly)
+#never actually gets here
+stop("we need to quit because something was seriously wrong.  Please look at the results")} 
+}
+
+#the main funcion starts here
 #if(!require(mnormt) ) {stop("I am sorry, you must have mnormt installed to use polychoric")}
 #if(polycor && (!require(polycor))) {warning ("I am sorry, you must have  polycor installed to use polychoric with the polycor option")
 # polycor <- FALSE}
@@ -211,10 +259,13 @@ if((prod(dim(x)) == 4) | is.table(x))  {result <- polytab(x,correct=correct)
 x <- as.matrix(x)
 if(!is.numeric(x)) {x <- matrix(as.numeric(x),ncol=nvar)
     message("Converted non-numeric input to numeric")}
-#xt <- table(x)   #this is clearly not a good idea
-#nvalues <- length(xt)  #find the number of response alternatives 
-
-
+# xt <- table(x)   #this finds the number of alternatives in all of x
+# nvalues <- length(xt)  #find the number of response alternatives 
+# maxx <- max(x,na.rm=TRUE) 
+# if (maxx > nvalues)  {#now, if max(xt) > nvalues we need to recode this to range from 1 to nvalues  #added Jan 10, 2018
+#    xtvalues <-as.numeric(names(xt))
+#    for(i in 1:nvalues) {x[x==xtvalues[i]] <- i} 
+#    }
 nvalues <- max(x,na.rm=TRUE) - min(x,na.rm=TRUE) + 1
 if(nvalues > 8) stop("You have more than 8 categories for your items, polychoric is probably not needed")
  #first delete any bad cases
@@ -253,7 +304,7 @@ colnames(tau) <- colnames(x)
 mat <- matrix(0,nvar,nvar)
 colnames(mat) <- rownames(mat) <- colnames(x)
 #x <- x - min(x,na.rm=TRUE) +1  #this is essential to get the table function to order the data correctly -- but we have already done it
-
+if(is.null(y)) {
 mat <- matpLower(x,nvar,gminx,gmaxx,gminy,gmaxy)  #the local copy has the extra paremeters   #do the multicore version
 
 
@@ -264,12 +315,41 @@ mat <- matpLower(x,nvar,gminx,gmaxx,gminy,gmaxy)  #the local copy has the extra
 
  colnames(mat) <- rownames(mat) <- colnames(x)
  tau <- t(tau)
-  result <- list(rho = mat,tau = tau,n.obs=nsub,Call=cl) 
+ tauy<- NULL
+ } else { #process the x * y data
+   ymin <- apply(y,2,function(x) min(x,na.rm=TRUE))
+   ymin <- min(ymin,na.rm=TRUE)
+   
+   nvar.y <- NCOL(y)
+   y <- t((t(y) - ymin +1))  #all numbers go from 1 to ymax +1
+   ymax <- apply(y,2,function(x) max(x,na.rm=TRUE))
+   ymax <- max(ymax,na.rm=TRUE)
+   gminy <- 1
+   gmaxy <- ymax 
+   nvaluesy <- ymax - ymin +1
+   yfreq <- apply(y,2,tabulate,nbins=nvaluesy)
+ 	n.obs.y <- colSums(yfreq)
+	yfreq <- t(t(yfreq)/n.obs.y)
+	tauy <- qnorm(apply(yfreq,2,cumsum))[1:(nvalues-1),]
+	if(!is.matrix(tauy)) tauy <- matrix(tauy,ncol=nvar.y) 
+	rownames(tauy) <- 1:(nvalues-1)
+	colnames(tauy) <- colnames(y)
+	
+   mat <- matpxy(x,y,nvar,nvar.y,gminx,gmaxx,gminy,gmaxy,tauy)
+   mat <- matrix(mat,ncol=nvar,nrow=nvar.y)
+   colnames(mat )<- colnames(x)
+   rownames(mat) <- colnames(y)
+   tauy <- t(tauy)
+   tau <- t(tau)
+   }
+  result <- list(rho = mat,tau = tau,tauy = tauy,n.obs=nsub,Call=cl) 
   
  class(result) <- c("psych","poly")
  }
   return(result) 
   }
+  
+  #####
 
 
 
@@ -296,6 +376,7 @@ function(tab,correct=TRUE) {
 
 #9/6/14  to facilitate mixed cor  we find polytomous by dichotomous correlations
 #4/08/17  fixed to not do table(p) or table(d)  
+#has a problem if we are correcting 0 values 
 "polydi" <- function(p,d,taup,taud,global=TRUE,ML = FALSE, std.err = FALSE,weight=NULL,progress=TRUE,na.rm=TRUE,delete=TRUE,correct=.5) {
 
 #if(!require(parallel)) {message("polychoric requires the parallel package.")}
@@ -312,7 +393,6 @@ for(i in 1:np) {for (j in 1:nd) {
    jl [k] <- j
    k <- k+1}
    }
-
 poly <- mcmapply(function(i,j) myfun(x,i,j,correct=correct,taup=taup,taud=taud,gminx=gminx,gmaxx=gmaxx,gminy=gminy,gmaxy=gmaxy,np=np) , il,jl+np)  #the multicore version
 #poly <- mapply(function(i,j) myfun(x,i,j,correct=correct,taup=taup,taud=taud,gminx=gminx,gmaxx=gmaxx,gminy=gminy,gmaxy=gmaxy,np=np) , il,jl +np)   #the normal version for debugging
 #now make it a matrix
@@ -337,8 +417,12 @@ d <- as.matrix(d)
 #nvalues <- length(xt)  #find the number of response alternatives 
 nvalues <- max(p,na.rm=TRUE) - min(p,na.rm=TRUE) + 1
 #dt <- table(d)
-dvalues <- max(d,na.rm=TRUE) - min(d,na.rm=TRUE) + 1
-if(dvalues !=2) stop("You did not supply a dichotomous variable")
+dmin <- apply(d,2,function(x) min(x,na.rm=TRUE))
+dmax <- apply(d,2,function(x) max(x,na.rm=TRUE))
+dvalues <- max(dmax-dmin) 
+
+
+if(dvalues !=1) stop("You did not supply a dichotomous variable")
 if(nvalues > 8) stop("You have more than 8 categories for your items, polychoric is probably not needed")
  #first delete any bad cases
   item.var <- apply(p,2,sd,na.rm=na.rm)
@@ -353,17 +437,17 @@ pmin <- apply(p,2,function(x) min(x,na.rm=TRUE))  #allow for different minima
 minx <- min(pmin)
 p <- t(t(p) - pmin +1)  #all numbers now go from 1 to nvalues 
 #p <- t(t(p) - gminx +1)  #all numbers now go from 1 to nvalues    but we should use global minimima
-dmin <- apply(d,2,function(x) min(x,na.rm=TRUE))
+
 #gminy <- min(dmin)
 miny <- min(dmin)
 #d <-  t(t(d) - gminy +1)
-d <-  t(t(d) - dmin +1)
+d <-  t(t(d) - dmin +1)   #this allows a separate minimum for each d variable
 gminx <- gminy <- 1    #set the global minima to 1
 
 pmax <- apply(p,2,function(x)  max(x,na.rm=TRUE)) #check for different maxima
 gmaxx <- max(pmax)
-if (min(pmax) != max(pmax)) {#global <- FALSE
-                      warning("The items do not have an equal number of response alternatives, I am using the largest number of categories in the polytomous set")}
+if (min(pmax) != max(pmax)) {global <- FALSE
+          warning("The items do not have an equal number of response alternatives, I am setting global to FALSE")}
 gmaxy <- max(apply(d,2,function(x) max(x,na.rm=TRUE)))                      
 #xfreq <- apply(x- xmin + 1,2,tabulate,nbins=nvalues)
 pfreq <- apply(p,2,tabulate,nbins=nvalues)
@@ -386,6 +470,7 @@ rownames(mat) <- colnames(p)
 colnames(mat)  <- colnames(d)
 #x <- x - min(x,na.rm=TRUE) +1  #this is essential to get the table function to order the data correctly
 x <- cbind(p,d)
+
 mat <- matpLower(x,np,nd,taup,taud,gminx,gmaxx,gminy,gmaxy)  #the local copy has the extra paremeters   #do the multicore version
 
  
