@@ -29,7 +29,8 @@ if(is.null(uniq)) {diag(model) <- 1 } else { diag(model) <- uniq  + diag(model)}
                                       observed <- t( eX$vectors %*% diag(sqrt(pmax(eX$values, 0)), nvar) %*%  t(observed) + mu) 
                                       theta <- observed
   	if(items) {observedp <- matrix(t(pnorm(a*t(observed)- d)),n,nvar) 
-  	         observed[] <- rbinom(n*nvar, cat, observedp)}
+  	         #observed[] <- rbinom(n*nvar, cat, observedp) #this puts in error again
+  	         observed[] <- round(cat * observedp)}
   	  colnames(observed) <- colnames(model)
   r <- cor(observed) 
   } 
@@ -149,8 +150,9 @@ ev <- eigen(observed.cor)$values
 f1 <- fa(observed.cor)$loadings
 om.fa <- sum(f1)^2/sum(observed.cor)
 e.f1 <- sum(f1^2)/nvar
-    sem.model <- omega.sem(x$fload,sl=TRUE,nf=nfact)  #this is the model based upon the true values
-    if(sem) {if(!requireNamespace('sem')) {stop("You must have the sem package installed to use omegaSem")} else {sem.om <- try(sem(model=sem.model,S=observed.cor, N=n))} 
+    sem.model <- omegaSem(x$fload,sl=TRUE,nfactors=nfact)  #this is the model based upon the true values
+    if(sem) {stop('The option to use the sem package  has been replaced with calls to lavaan')
+    if(!requireNamespace('sem')) {stop("You must have the sem package installed to use omegaSem")} else {sem.om <- try(sem(model=sem.model,S=observed.cor, N=n))} 
  
   omega.cfa <- omegaFromSem(observed.cor,sem.om,flip=flip)
     if(omega.cfa$omega >1) omega.cfa$omega <- NA
@@ -162,11 +164,11 @@ if(n > 0) {
 om <- try(omega(x$observed,om.fact,flip=flip,plot=FALSE,option=option))}
           ic <- suppressWarnings(ICLUST(x$observed,1,plot=FALSE))} else {
            if (sem) {om <- try(omegaSem(x$model,om.fact,flip=flip,plot=FALSE,option=option))} else {om <- try(omega(x$model,om.fact,flip=flip,plot=FALSE,option=option))
-           if(class(om)=="try-error") {message("Error in sem. iteration = ",i)
+           if(inherits(om,"try-error")) {message("Error in sem. iteration = ",i)
             om <- NA
             next}}
           ic <- suppressWarnings(ICLUST(x$model,1,plot=FALSE))}
-results
+#results
 if(sem) {results[i,"omega"] <- om$omegaSem$omega_h
                 loads <- om$omegaSem$schmid$sl
      
